@@ -1,10 +1,11 @@
 #!/bin/false
-# ABSTRACT: MAC address functions and object
+# ABSTRACT: MAC hardware address functions and object (EUI48 and EUI64)
 
 use strict;
 use warnings;
+use v5.10;
 package NetAddr::MAC;
-$NetAddr::MAC::VERSION = '0.94';
+$NetAddr::MAC::VERSION = '0.98';
 
 use Carp qw( croak );
 use List::Util qw( first );
@@ -38,7 +39,7 @@ use constant ETHER2TOKEN => (
 );
 
 use base qw( Exporter );
-use vars qw( %EXPORT_TAGS @EXPORT_OK );
+use vars qw( %EXPORT_TAGS );
 
 %EXPORT_TAGS = (
     all => [
@@ -81,158 +82,17 @@ use vars qw( %EXPORT_TAGS @EXPORT_OK );
 
 Exporter::export_ok_tags( keys %EXPORT_TAGS );
 
-=encoding utf8
-=head1 NAME
-
-NetAddr::MAC - Handles hardware MAC Addresses (EUI-48 and EUI-64)
-
-=head1 VERSION
-
-version 0.94
-
-=head1 SYNOPSIS
-
-    use NetAddr::MAC;
-
-    my $mac = NetAddr::MAC->new( '00:11:22:aa:bb:cc' );
-    my $mac = NetAddr::MAC->new( mac => '0011.22AA.BBCC' );
-
-    print "MAC provided at object creation was: ", $mac->original;
-
-    print "EUI48\n" if $mac->is_eui48;
-    print "EUI64\n" if $mac->is_eui64;
-
-    print "Unicast\n" if $mac->is_unicast;
-    print "Multicast\n" if $mac->is_multicast;
-    print "Broadcast\n" if $mac->is_broadcast;
-
-    print "Locally Administerd\n" if $mac->is_local;
-    print "Universally Administered\n" if $mac->is_universal;
-
-    print 'Basic Format: ',$mac->as_basic,"\n";
-    print 'Bpr Format: ',  $mac->as_bpr,"\n";
-    print 'Cisco Format: ',$mac->as_cisco,"\n";
-    print 'IEEE Format: ', $mac->as_ieee,"\n";
-    print 'IPv6 Address: ',$mac->as_ipv6_suffix,"\n";
-    print 'Microsoft Format: ',$mac->as_microsoft,"\n";
-    print 'Single Dash Format: ',$mac->as_singledash,"\n";
-    print 'Sun Format: ',  $mac->as_sun,"\n";
-    print 'Token Ring Format: ', $mac->as_tokenring,"\n";
-
-
-    use NetAddr::MAC qw( :all );
-
-    my $mac = q/00.11.22.33.44.55/;
-
-    print "EUI48\n" if mac_is_eui48($mac);
-    print "EUI64\n" if mac_is_eui64($mac);
-
-    print "Unicast\n" if mac_is_unicast($mac);
-    print "Multicast\n" if mac_is_multicast($mac);
-    print "Broadcast\n" if mac_is_broadcast($mac);
-
-    print "Locally Administerd\n" if mac_is_local($mac);
-    print "Universally Administered\n" if mac_is_universal($mac);
-
-    print 'Basic Format: ',mac_as_basic($mac),"\n";
-    print 'Bpr Format: ',  mac_as_bpr($mac),"\n";
-    print 'Cisco Format: ',mac_as_cisco($mac),"\n";
-    print 'IEEE Format: ', mac_as_ieee($mac),"\n";
-    print 'IPv6 Address: ',mac_as_ipv6_suffix($mac),"\n";
-    print 'Microsoft Format: ',mac_as_microsoft($mac),"\n";
-    print 'Single Dash Format: ', mac_as_singledash($mac),"\n";
-    print 'Sun Format: ',  mac_as_sun($mac),"\n";
-    print 'Token Ring Format: ',mac_as_tokenring($mac),"\n";
-
-=head1 DESCRIPTION
-
-This module provides an interface to deal with Media Access Control (or MAC)
-addresses.  These are the addresses that uniquely identify a device on
-various layer 2 networks. Although the most common case is hardware addresses
-on Ethernet network cards, there are a variety of devices that use this
-system of addressing.
-
-This module supports both EUI-48 and EUI-64 addresses and implements an
-OO and a functional interface.
-
-Some networks that use EUI-48 (or MAC-48) addresses include:
-
-    Ethernet
-    802.11 wireless networks
-    Bluetooth
-    IEEE 802.5 token ring
-    FDDI
-    ATM
-
-Some networks that use EUI-64 addresses include:
-
-    Firewire
-    IPv6 (sort of)
-    ZigBee / 802.15.4 wireless personal-area networks
-
-
-=head1 OO METHODS
-
-=head2 NetAddr::MAC->new( mac => $mac )
-
-Creates and returns a new NetAddr::MAC object.  The MAC value is required.
-
-=head2 NetAddr::MAC->new( mac => $mac, %options )
-
-As above, but %options may include any or none of the following
-
-=over 4
-
-=item * die_on_error
-
-If set to true, errors will result in a die (croak) rather than populating I<$errstr>
-
-B<Take care when using the mac_is_* functions!> they will return false in both
-the case of an error and according to the properties of the MAC address. You will
-therefore need to enable die_on_error or check I<$errstr> when false is returned.
-
-=item * priority
-
-This is the bridge priority as an integer, which can also be set by providing a mac
-address in the following format, where 60 is the priority.
-
- 60#0011.22aa.bbcc
-
-This is a cisco thing, so typically the above is the format you would see. But we
-are flexible enough to handle formats like...
-
- 60#00:11:22:aa:bb:cc
- 60#001122aabbcc
- 60#00-11-22-aa-bb-cc
- etc.
-
-If priority is provided as an option and as part of the mac address string, an
-error will occur only if they differ.
-
-Priority defaults to 0 if not provided.
-
-=back
-
-=head2 NetAddr::MAC->new( $mac )
-
-Simplified creation method
-
-=head2 NetAddr::MAC->new( $mac, %options )
-
-As above but with %options
-
-=cut
 
 sub new {
 
-    my ( $p, @a ) = @_;
+    my ( $p, @q ) = @_;
     my $c = ref($p) || $p;
     my $self = bless {}, $c;
 
     # clear the errstr, see also RT96045
     $NetAddr::MAC::errstr = undef;
 
-    unless (@a) {
+    unless (@q) {
         my $e = q|Please provide a mac address|;
         croak "$e\n" if $NetAddr::MAC::die_on_error;
         $NetAddr::MAC::errstr = $e;
@@ -240,7 +100,7 @@ sub new {
     }
 
     # massage a single argument into a mac argument if needed
-    $self->_init( @a % 2 ? ( mac => shift @a, @a ) : @a )
+    $self->_init( @q % 2 ? ( mac => shift @q, @q ) : @q )
       or return;
 
     return $self
@@ -338,7 +198,7 @@ sub new {
                                       : $o
                 } @parts;
 
-            # 12 characters for EUI-48, 16 for EUI-64
+            # 12 characters for EUI48, 16 for EUI64
             if (
                 @parts == 1
                 && (   length $parts[0] == EUI48LENGTHHEX
@@ -392,11 +252,33 @@ sub new {
 
 }
 
-=head2 original
 
-returns the original B<mac> string as used when creating the MAC object
+sub random {
+
+    my ( $p, @q ) = @_;
+    my $c = ref($p) || $p;
+    my $self = bless {}, $c;
+
+    # clear the errstr, see also RT96045
+    $NetAddr::MAC::errstr = undef;
+
+    unless (@q) {
+        my $e = q|Please provide an oui prefix|;
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return
+    }
+
+    die 'TODO';
+
+    # massage a single argument into a mac argument if needed
+    $self->_init( @q % 2 ? ( oui => shift @q, @q ) : @q )
+      or return;
+
+    return $self
+
+}
 
-=cut
 
 sub original {
 
@@ -405,14 +287,6 @@ sub original {
 
 }
 
-=head2 oui
-
-returns the mac address's Organizationally Unique Identifier (OUI) with dashes
-in Hexadecimal / Canonical format:
-
- AC-DE-48
-
-=cut
 
 sub oui {
 
@@ -425,20 +299,6 @@ sub oui {
 
 }
 
-=head2 errstr
-
-returns the error (if one occured).
-
-This is intended for use with the object. Its not exported at all.
-
-Note: this method is used once the NetAddr::MAC object is successfully
-created. For now the to_eui48 method is the only method that will
-return an error once the object is created.
-
-When creating objects, you will need to catch errors with either the
-I<or> function, or the I<eval> way.
-
-=cut
 
 sub errstr {
 
@@ -448,35 +308,18 @@ sub errstr {
 
 }
 
-=head1 OO PROPERTY METHODS
-
-=head2 is_eui48
-
-returns true if mac address is determined to be of the EUI48 standard
-
-=cut
 
 sub is_eui48 {
     my $self = shift;
     return scalar @{ $self->{mac} } == EUI48LENGTHDEC
 }
 
-=head2 is_eui64
-
-returns true if mac address is determined to be of the EUI64 standard
-
-=cut
 
 sub is_eui64 {
     my $self = shift;
     return scalar @{ $self->{mac} } == EUI64LENGTHDEC
 }
 
-=head2 is_multicast
-
-returns true if mac address is determined to be a multicast address
-
-=cut
 
 sub is_multicast {
     my $self = shift;
@@ -485,11 +328,6 @@ sub is_multicast {
 }
 
 
-=head2 is_broadcast
-
-returns true if mac address is determined to be a broadcast address
-
-=cut
 
 sub is_broadcast {
     my $self = shift;
@@ -500,17 +338,6 @@ sub is_broadcast {
     return 1
 }
 
-=head2 is_vrrp
-
-returns true if mac address is determined to be a Virtual Router Redundancy (VRRP) address
-
-ie. 00-00-5E-00-01-XX
-
-always returns false for eui64.
-
-I'm not quite sure what to do with 01-00-5E-00-00-12, suggestions welcomed.
-
-=cut
 
 sub is_vrrp {
     my $self = shift;
@@ -525,15 +352,6 @@ sub is_vrrp {
 
 }
 
-=head2 is_hsrp
-
-returns true if mac address is determined to be a Hot Standby Router (HSRP) address
-
-ie. 00-00-0C-07-AC-XX
-
-always returns false for eui64.
-
-=cut
 
 sub is_hsrp {
     my $self = shift;
@@ -548,15 +366,6 @@ sub is_hsrp {
 
 }
 
-=head2 is_hsrp2
-
-returns true if mac address is determined to be a Hot Standby Router Version 2 (HSRPv2) address
-
-ie. 00-00-0C-9F-FX-XX
-
-always returns false for eui64.
-
-=cut
 
 sub is_hsrp2 {
     my $self = shift;
@@ -572,19 +381,6 @@ sub is_hsrp2 {
 }
 
 
-=head2 is_msnlb
-
-returns true if mac address is determined to be a MS Network Load Balancing MAC address
-
-ie. 02-BF-1-2-3-4 for unicast or 03-BF-1-2-3-4 for multicast
-
-where 1-2-3-4 is the clusters primary IP address
-
-for outbound packets, clusters members will send from 02-n-1-2-3-4 where n is the node priorty. this function does NOT return true for those adddresses.
-
-always returns false for eui64.
-
-=cut
 
 sub is_msnlb {
     my $self = shift;
@@ -597,61 +393,30 @@ sub is_msnlb {
 
 }
 
-=head2 is_unicast
-
-returns true if mac address is determined to be a unicast address
-
-=cut
 
 sub is_unicast {
     my $self = shift;
     return ! ($self->{mac}->[0] & 1);
 }
 
-=head2 is_local
-
-returns true if mac address is determined to be locally administered
-
-=cut
 
 sub is_local {
     my $self = shift;
     return $self->{mac}->[0] & 2
 }
 
-=head2 is_universal
-
-returns true if mac address is determined to be universally administered
-
-=cut
 
 sub is_universal {
     my $self = shift;
     return !is_local($self)
 }
 
-=head1 OO NORMALIZATION METHODS
-
-=head2 as_basic
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and without delimiters
-
- 001122aabbcc
-
-=cut
 
 sub as_basic {
     my $self = shift;
     return join( q{}, map { sprintf( '%02x', $_ ) } @{ $self->{mac} } )
 }
 
-=head2 as_bridge_id
-
-returns mac address with the priority, a hash, then the mac address normalized with I<as_cisco>
-
- 45#0011.22aa.bbcc
-
-=cut
 
 sub as_bridge_id {
     my $self = shift;
@@ -660,14 +425,6 @@ sub as_bridge_id {
         . $self->as_cisco;
 }
 
-=head2 as_bpr
-
-returns the mac address normalized as a hexidecimal string that is 0 padded with B<:> delimiters and with
-B<1,length> leading where I<length> is the number of hex pairs (ie 6 for EUI48)
-
- 1,6,00:11:22:aa:bb:cc
-
-=cut
 
 sub as_bpr {
     my $self = shift;
@@ -677,14 +434,6 @@ sub as_bpr {
       . join( q{:}, map { sprintf( '%02x', $_ ) } @{ $self->{mac} } );
 }
 
-=head2 as_cisco
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and with B<.> delimiting every 2nd octet
-(ie after every 4th character)
-
- 0011.22aa.bbcc
-
-=cut
 
 sub as_cisco {
     my $self = shift;
@@ -693,25 +442,12 @@ sub as_cisco {
           join( q{}, map { sprintf( '%02x', $_ ) } @{ $self->{mac} } ) )
 }
 
-=head2 as_ieee
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and with B<:> delimiting every octet
-(ie after every 2nd character)
-
- 00:34:56:78:9a:bc
-
-=cut
 
 sub as_ieee {
     my $self = shift;
     return join( q{:}, map { sprintf( '%02x', $_ ) } @{ $self->{mac} } )
 }
 
-=head2 as_ipv6_suffix
-
-returns the EUI-64 address in the format used for an IPv6 autoconf address suffix
-
-=cut
 
 sub as_ipv6_suffix {
 
@@ -743,28 +479,12 @@ sub as_ipv6_suffix {
     );
 }
 
-=head2 as_microsoft
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character)
-
- 00-34-56-78-9a-bc
-
-=cut
 
 sub as_microsoft {
     my $self = shift;
     return join( q{-}, map { sprintf( '%02x', $_ ) } @{ $self->{mac} } )
 }
 
-=head2 as_pgsql
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and has a I<:> in the middle of the hex string.
-this appears in the pgsql documentation along with the single dash version
-
- 001122:334455
-
-=cut
 
 sub as_pgsql {
     my $self = shift;
@@ -782,14 +502,6 @@ sub as_pgsql {
     );
 }
 
-=head2 as_singledash
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and has a dash in the middle of the hex string.
-this appears in the pgsql documentation.
-
- 001122-334455
-
-=cut
 
 sub as_singledash {
     my $self = shift;
@@ -807,28 +519,12 @@ sub as_singledash {
     );
 }
 
-=head2 as_sun
-
-returns the mac address normalized as a hexidecimal string that is B<not> padded and with B<-> delimiting every octet
-(ie after every 2nd character)
-
- 0-34-56-78-9a-bc
-
-=cut
 
 sub as_sun {
     my $self = shift;
     return join( q{-}, map { sprintf( '%01x', $_ ) } @{ $self->{mac} } )
 }
 
-=head2 as_tokenring
-
-returns the mac address normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character) and each octect is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
-
- 00-2d-6a-1e-59-3d
-
-=cut
 
 sub as_tokenring {
 
@@ -836,14 +532,6 @@ sub as_tokenring {
     return join( q{-}, map { (ETHER2TOKEN)[$_] } @{ $self->{mac} } )
 }
 
-=head2 to_eui48
-
-converts to EUI-48 (if the eui-64 was derived from eui-48)
-
-this function will fail if the mac was not derived from eui-48.
-you will need to catch it and inspect the error message.
-
-=cut
 
 sub to_eui48 {
 
@@ -870,12 +558,6 @@ sub to_eui48 {
     return 1
 }
 
-=head2 to_eui64
-
-converts to EUI-64, or in other words encapsulates EUI-48 to become EUI-64
-if needed
-
-=cut
 
 sub to_eui64 {
 
@@ -884,7 +566,7 @@ sub to_eui64 {
     # be slightly evil here so that hashrefs and objects work
     if ( is_eui48($self) ) {
 
-        # convert to eui-64
+        # convert to EUI64
         $self->{mac} = [
             @{ $self->{mac} }[ 0 .. 2 ],
             0xff,
@@ -898,13 +580,6 @@ sub to_eui64 {
     return 1
 }
 
-=head1 PROCEDURAL PROPERTY FUNCTIONS
-
-=head2 mac_is_eui48($mac)
-
-returns true if mac address in $mac is determined to be of the EUI48 standard
-
-=cut
 
 sub mac_is_eui48 {
 
@@ -923,11 +598,6 @@ sub mac_is_eui48 {
 
 }
 
-=head2 mac_is_eui64($mac)
-
-returns true if mac address in $mac is determined to be of the EUI64 standard
-
-=cut
 
 sub mac_is_eui64 {
 
@@ -946,11 +616,6 @@ sub mac_is_eui64 {
 
 }
 
-=head2 mac_is_multicast($mac)
-
-returns true if mac address in $mac is determined to be a multicast address
-
-=cut
 
 sub mac_is_multicast {
 
@@ -970,11 +635,6 @@ sub mac_is_multicast {
 }
 
 
-=head2 mac_is_broadcast($mac)
-
-returns true if mac address in $mac is determined to be a broadcast address
-
-=cut
 
 sub mac_is_broadcast {
 
@@ -993,11 +653,6 @@ sub mac_is_broadcast {
 
 }
 
-=head2 mac_is_unicast($mac)
-
-returns true if mac address in $mac is determined to be a unicast address
-
-=cut
 
 sub mac_is_unicast {
 
@@ -1016,13 +671,6 @@ sub mac_is_unicast {
 
 }
 
-=head2 mac_is_vrrp($mac)
-
-returns true if mac address is $mac is determined to be a Virtual Router Redundancy (VRRP) address
-
-ie. 00-00-5E-00-01-XX
-
-=cut
 
 sub mac_is_vrrp {
 
@@ -1043,13 +691,6 @@ sub mac_is_vrrp {
 }
 
 
-=head2 mac_is_hsrp($mac)
-
-returns true if mac address is $mac is determined to be a Hot Standby Router (HSRP) address
-
-ie. 00-00-0C-07-AC-XX
-
-=cut
 
 sub mac_is_hsrp {
 
@@ -1069,13 +710,6 @@ sub mac_is_hsrp {
 
 }
 
-=head2 mac_is_hsrp2($mac)
-
-returns true if mac address is $mac is determined to be a Hot Standby Router Version 2 (HSRPv2) address
-
-ie. 00-00-0C-9F-FX-XX
-
-=cut
 
 sub mac_is_hsrp2 {
 
@@ -1095,13 +729,6 @@ sub mac_is_hsrp2 {
 
 }
 
-=head2 mac_is_msnlb($mac)
-
-returns true if mac address is $mac is determined to be a MS Network Load Balancing address
-
-ie. 02-BF-XX-XX-XX-XX or 03-BF-XX-XX-XX-XX
-
-=cut
 
 sub mac_is_msnlb {
 
@@ -1121,11 +748,6 @@ sub mac_is_msnlb {
 
 }
 
-=head2 mac_is_local($mac)
-
-returns true if mac address in $mac is determined to be locally administered
-
-=cut
 
 sub mac_is_local {
 
@@ -1144,11 +766,6 @@ sub mac_is_local {
 
 }
 
-=head2 mac_is_universal($mac)
-
-returns true if mac address in $mac is determined to be universally administered
-
-=cut
 
 sub mac_is_universal {
 
@@ -1167,15 +784,6 @@ sub mac_is_universal {
 
 }
 
-=head1 PROCEDURAL NORMALIZATION METHODS
-
-=head2 mac_as_basic($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and without delimiters
-
- 001122aabbcc
-
-=cut
 
 sub mac_as_basic {
 
@@ -1194,14 +802,6 @@ sub mac_as_basic {
 
 }
 
-=head2 mac_as_bpr($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded, with B<:> delimiting and
-B<1,length> leading. I<length> is the number of hex pairs (6 for EUI48)
-
- 1,6,00:11:22:aa:bb:cc
-
-=cut
 
 sub mac_as_bpr {
 
@@ -1220,14 +820,6 @@ sub mac_as_bpr {
 
 }
 
-=head2 mac_as_cisco($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and with B<.> delimiting every 2nd octet
-(ie after every 4th character)
-
- 0011.22aa.bbcc
-
-=cut
 
 sub mac_as_cisco {
 
@@ -1246,14 +838,6 @@ sub mac_as_cisco {
 
 }
 
-=head2 mac_as_ieee($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and with B<:> delimiting every octet
-(ie after every 2nd character)
-
- 00:34:56:78:9a:bc
-
-=cut
 
 sub mac_as_ieee {
 
@@ -1272,13 +856,6 @@ sub mac_as_ieee {
 
 }
 
-=head2 mac_as_ipv6_suffix($mac)
-
-returns the mac address in $mac in the format used for an IPv6 autoconf address suffix
-
-will convert from eui48 or eui64 if needed
-
-=cut
 
 sub mac_as_ipv6_suffix {
 
@@ -1297,14 +874,6 @@ sub mac_as_ipv6_suffix {
 
 }
 
-=head2 mac_as_microsoft($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character)
-
- 00-34-56-78-9a-bc
-
-=cut
 
 sub mac_as_microsoft {
 
@@ -1324,14 +893,6 @@ sub mac_as_microsoft {
 
 }
 
-=head2 mac_as_pgsql($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and a single B<:> delimiter
-in the middle. this format appears in their documenation, along with single dash version
-
- 003456:789abc
-
-=cut
 
 sub mac_as_pgsql {
 
@@ -1351,14 +912,6 @@ sub mac_as_pgsql {
 
 }
 
-=head2 mac_as_singledash($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and has a dash in the middle of the hex string.
-this appears in the pgsql documenation
-
- 001122-334455
-
-=cut
 
 sub mac_as_singledash {
 
@@ -1378,14 +931,6 @@ sub mac_as_singledash {
 
 }
 
-=head2 mac_as_sun($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is B<not> padded and with B<-> delimiting every octet
-(ie after every 2nd character)
-
- 0-34-56-78-9a-bc
-
-=cut
 
 sub mac_as_sun {
 
@@ -1405,14 +950,6 @@ sub mac_as_sun {
 
 }
 
-=head2 mac_as_tokenring($mac)
-
-returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character) and each octect is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
-
- 00-2d-6a-1e-59-3d
-
-=cut
 
 sub mac_as_tokenring {
 
@@ -1432,14 +969,496 @@ sub mac_as_tokenring {
 
 }
 
+
+1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+NetAddr::MAC - MAC hardware address functions and object (EUI48 and EUI64)
+
+=head1 VERSION
+
+version 0.98
+
+=head1 SYNOPSIS
+
+ use NetAddr::MAC;
+
+ my $mac = NetAddr::MAC->new( '00:11:22:aa:bb:cc' );
+ my $mac = NetAddr::MAC->new( mac => '0011.22AA.BBCC' );
+
+ print "MAC provided at object creation was: ", $mac->original;
+
+ print "EUI48\n" if $mac->is_eui48;
+ print "EUI64\n" if $mac->is_eui64;
+
+ print "Unicast\n" if $mac->is_unicast;
+ print "Multicast\n" if $mac->is_multicast;
+ print "Broadcast\n" if $mac->is_broadcast;
+
+ print "Locally Administered\n" if $mac->is_local;
+ print "Universally Administered\n" if $mac->is_universal;
+
+ print 'Basic Format: ',$mac->as_basic,"\n";
+ print 'Bpr Format: ',  $mac->as_bpr,"\n";
+ print 'Cisco Format: ',$mac->as_cisco,"\n";
+ print 'IEEE Format: ', $mac->as_ieee,"\n";
+ print 'IPv6 Address: ',$mac->as_ipv6_suffix,"\n";
+ print 'Microsoft Format: ',$mac->as_microsoft,"\n";
+ print 'Single Dash Format: ',$mac->as_singledash,"\n";
+ print 'Sun Format: ',  $mac->as_sun,"\n";
+ print 'Token Ring Format: ', $mac->as_tokenring,"\n";
+
+
+ use NetAddr::MAC qw( :all );
+
+ my $mac = q/00.11.22.33.44.55/;
+
+ print "EUI48\n" if mac_is_eui48($mac);
+ print "EUI64\n" if mac_is_eui64($mac);
+
+ print "Unicast\n" if mac_is_unicast($mac);
+ print "Multicast\n" if mac_is_multicast($mac);
+ print "Broadcast\n" if mac_is_broadcast($mac);
+
+ print "Locally Administerd\n" if mac_is_local($mac);
+ print "Universally Administered\n" if mac_is_universal($mac);
+
+ print 'Basic Format: ',mac_as_basic($mac),"\n";
+ print 'Bpr Format: ',  mac_as_bpr($mac),"\n";
+ print 'Cisco Format: ',mac_as_cisco($mac),"\n";
+ print 'IEEE Format: ', mac_as_ieee($mac),"\n";
+ print 'IPv6 Address: ',mac_as_ipv6_suffix($mac),"\n";
+ print 'Microsoft Format: ',mac_as_microsoft($mac),"\n";
+ print 'Single Dash Format: ', mac_as_singledash($mac),"\n";
+ print 'Sun Format: ',  mac_as_sun($mac),"\n";
+ print 'Token Ring Format: ',mac_as_tokenring($mac),"\n";
+
+=head1 DESCRIPTION
+
+This module provides an OO and functional interface to deal with Media Access
+Control (or MAC) addresses. These are the addresses that uniquely identify a
+device on various layer 2 networks. Although the most common case is hardware
+addresses on Ethernet network cards, there are a variety of devices that use
+this system of addressing.
+
+Both Extended Unique Identifier 48 and 64 addresses are supported.
+
+Some networks that use Extended Unique Identifier 48 (or MAC48) addresses include:
+
+ Ethernet
+ 802.11 wireless networks
+ Bluetooth
+ IEEE 802.5 token ring
+ FDDI
+ ATM
+
+Some networks that use Extended Unique Identifier 64 addresses include:
+
+ Firewire
+ IPv6 (sort of)
+ ZigBee / 802.15.4 wireless personal-area networks
+
+=for stopwords soe todo
+cisco errstr oui
+fx hsrp HSRPv2 vrrp autoconf
+pgsql unicast
+eui48 eui64
+4th vmware xen
+
+=head1 OO METHODS
+
+=head2 new
+
+The B<new> method creates instances of this object. It can be called
+in any of the following ways.
+
+=head3 NetAddr::MAC->new( mac => $mac )
+
+Creates and returns a new NetAddr::MAC object.  The MAC value is required.
+
+=head3 NetAddr::MAC->new( mac => $mac, %options )
+
+As above, but %options may include any or none of the following
+
+=over 4
+
+=item * die_on_error
+
+If set to true, errors will result in a die (croak) rather than populating I<$errstr>
+
+B<Take care when using the mac_is_* functions!> they will return false in both
+the case of an error and according to the properties of the MAC address. You will
+therefore need to enable die_on_error or check I<$errstr> when false is returned.
+
+=item * priority
+
+This is the bridge priority as an integer, which can also be set by providing a mac
+address in the following format, where 60 is the priority.
+
+ 60#0011.22aa.bbcc
+
+This is a cisco thing, so typically the above is the format you would see. But we
+are flexible enough to handle formats like...
+
+ 60#00:11:22:aa:bb:cc
+ 60#001122aabbcc
+ 60#00-11-22-aa-bb-cc
+ etc.
+
+If priority is provided as an option and as part of the mac address string, an
+error will occur only if they differ.
+
+Priority defaults to 0 if not provided.
+
+=back
+
+=head3 NetAddr::MAC->new( $mac )
+
+Simplified creation method
+
+=head3 NetAddr::MAC->new( $mac, %options )
+
+As above but with %options
+
+=head2 random
+
+As an alternative to L</new>, a "random" mac access based upon the provided
+B<oui> argument.
+
+Please consider the following information when selecting an OUI.
+
+If the first octal/digit/number is odd, then the MAC address L</is_multicast>
+
+OUI's used by virtualization software:
+
+Xen's prefix 00:16:3e
+VMware's prefix 00:50:56
+
+There are 4 sets of 'Locally Administered Address Ranges' that can be used
+without fear of conflict (from actual hardware):
+
+ x2-xx-xx-xx-xx-xx
+ x6-xx-xx-xx-xx-xx
+ xA-xx-xx-xx-xx-xx
+ xE-xx-xx-xx-xx-xx
+
+=head2 original
+
+Returns the original B<mac> string as used when creating the MAC object
+
+=head2 oui
+
+Returns the mac address's Organizationally Unique Identifier (OUI) with dashes
+in Hexadecimal / Canonical format:
+
+ AC-DE-48
+
+=head2 errstr
+
+Returns the error (if one occurred).
+
+This is intended for use with the object. Its not exported at all.
+
+Note: this method is used once the NetAddr::MAC object is successfully
+created. For now the to_eui48 method is the only method that will
+return an error once the object is created.
+
+When creating objects, you will need to catch errors with either the
+I<or> function, or the I<eval> way.
+
+=head1 OO PROPERTY METHODS
+
+=head2 is_eui48
+
+Returns true if mac address is determined to be of the EUI48 standard
+
+=head2 is_eui64
+
+Returns true if mac address is determined to be of the EUI64 standard
+
+=head2 is_multicast
+
+Returns true if mac address is determined to be a multicast address
+
+=head2 is_broadcast
+
+Returns true if mac address is determined to be a broadcast address
+
+=head2 is_vrrp
+
+Returns true if mac address is determined to be a Virtual Router Redundancy (VRRP) address
+
+i.e. 00-00-5E-00-01-XX
+
+always returns false for eui64.
+
+I'm not quite sure what to do with 01-00-5E-00-00-12, suggestions welcomed.
+
+=head2 is_hsrp
+
+Returns true if mac address is determined to be a Hot Standby Router (HSRP) address
+
+i.e. 00-00-0C-07-AC-XX
+
+always returns false for eui64.
+
+=head2 is_hsrp2
+
+Returns true if mac address is determined to be a Hot Standby Router Version 2 (HSRPv2) address
+
+i.e. 00-00-0C-9F-FX-XX
+
+always returns false for eui64.
+
+=head2 is_msnlb
+
+Returns true if mac address is determined to be a MS Network Load Balancing MAC address
+
+i.e. 02-BF-1-2-3-4 for unicast or 03-BF-1-2-3-4 for multicast
+
+where 1-2-3-4 is the clusters primary IP address
+
+for outbound packets, clusters members will send from 02-n-1-2-3-4 where n is the node priority. this function does NOT return true for those addresses.
+
+always returns false for eui64.
+
+=head2 is_unicast
+
+Returns true if mac address is determined to be a unicast address
+
+=head2 is_local
+
+Returns true if mac address is determined to be locally administered
+
+=head2 is_universal
+
+Returns true if mac address is determined to be universally administered
+
+=head1 OO NORMALIZATION METHODS
+
+=head2 as_basic
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and without delimiters
+
+ 001122aabbcc
+
+=head2 as_bridge_id
+
+Returns mac address with the priority, a hash, then the mac address normalized with I<as_cisco>
+
+ 45#0011.22aa.bbcc
+
+=head2 as_bpr
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded with B<:> delimiters and with
+B<1,length> leading where I<length> is the number of hex pairs (i.e. 6 for EUI48)
+
+ 1,6,00:11:22:aa:bb:cc
+
+=head2 as_cisco
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and with B<.> delimiting every 2nd octet
+(i.e. after every 4th character)
+
+ 0011.22aa.bbcc
+
+=head2 as_ieee
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and with B<:> delimiting every octet
+(i.e. after every 2nd character)
+
+ 00:34:56:78:9a:bc
+
+=head2 as_ipv6_suffix
+
+Returns the EUI64 address in the format used for an IPv6 autoconf address suffix
+
+=head2 as_microsoft
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and with B<-> delimiting every octet
+(i.e. after every 2nd character)
+
+ 00-34-56-78-9a-bc
+
+=head2 as_pgsql
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and has a I<:> in the middle of the hex string.
+this appears in the pgsql documentation along with the single dash version
+
+ 001122:334455
+
+=head2 as_singledash
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and has a dash in the middle of the hex string.
+this appears in the pgsql documentation.
+
+ 001122-334455
+
+=head2 as_sun
+
+Returns the mac address normalized as a hexadecimal string that is B<not> padded and with B<-> delimiting every octet
+(i.e. after every 2nd character)
+
+ 0-34-56-78-9a-bc
+
+=head2 as_tokenring
+
+Returns the mac address normalized as a hexadecimal string that is 0 padded and with B<-> delimiting every octet
+(i.e. after every 2nd character) and each octet is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
+
+ 00-2d-6a-1e-59-3d
+
+=head2 to_eui48
+
+Converts to EUI48 (if the EUI64 was derived from EUI48)
+
+This function will fail if the mac was not derived from EUI48.
+you will need to catch it and inspect the error message.
+
+=head2 to_eui64
+
+Converts to EUI64, or in other words encapsulates EUI48 to become EUI64
+if needed
+
+=head1 PROCEDURAL PROPERTY FUNCTIONS
+
+=head2 mac_is_eui48($mac)
+
+Returns true if mac address in $mac is determined to be of the EUI48 standard
+
+=head2 mac_is_eui64($mac)
+
+Returns true if mac address in $mac is determined to be of the EUI64 standard
+
+=head2 mac_is_multicast($mac)
+
+Returns true if mac address in $mac is determined to be a multicast address
+
+=head2 mac_is_broadcast($mac)
+
+Returns true if mac address in $mac is determined to be a broadcast address
+
+=head2 mac_is_unicast($mac)
+
+Returns true if mac address in $mac is determined to be a unicast address
+
+=head2 mac_is_vrrp($mac)
+
+Returns true if mac address is $mac is determined to be a Virtual Router Redundancy (VRRP) address
+
+i.e. 00-00-5E-00-01-XX
+
+=head2 mac_is_hsrp($mac)
+
+Returns true if mac address is $mac is determined to be a Hot Standby Router (HSRP) address
+
+i.e. 00-00-0C-07-AC-XX
+
+=head2 mac_is_hsrp2($mac)
+
+Returns true if mac address is $mac is determined to be a Hot Standby Router Version 2 (HSRPv2) address
+
+i.e. 00-00-0C-9F-FX-XX
+
+=head2 mac_is_msnlb($mac)
+
+Returns true if mac address is $mac is determined to be a MS Network Load Balancing address
+
+i.e. 02-BF-XX-XX-XX-XX or 03-BF-XX-XX-XX-XX
+
+=head2 mac_is_local($mac)
+
+Returns true if mac address in $mac is determined to be locally administered
+
+=head2 mac_is_universal($mac)
+
+Returns true if mac address in $mac is determined to be universally administered
+
+=head1 PROCEDURAL NORMALIZATION METHODS
+
+=head2 mac_as_basic($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and without delimiters
+
+ 001122aabbcc
+
+=head2 mac_as_bpr($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded, with B<:> delimiting and
+B<1,length> leading. I<length> is the number of hex pairs (6 for EUI48)
+
+ 1,6,00:11:22:aa:bb:cc
+
+=head2 mac_as_cisco($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and with B<.> delimiting every 2nd octet
+(i.e. after every 4th character)
+
+ 0011.22aa.bbcc
+
+=head2 mac_as_ieee($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and with B<:> delimiting every octet
+(i.e. after every 2nd character)
+
+ 00:34:56:78:9a:bc
+
+=head2 mac_as_ipv6_suffix($mac)
+
+Returns the mac address in $mac in the format used for an IPv6 autoconf address suffix
+
+It will convert from eui48 or eui64 if needed
+
+=head2 mac_as_microsoft($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and with B<-> delimiting every octet
+(i.e. after every 2nd character)
+
+ 00-34-56-78-9a-bc
+
+=head2 mac_as_pgsql($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and a single B<:> delimiter
+in the middle. this format appears in their documentation, along with single dash version
+
+ 003456:789abc
+
+=head2 mac_as_singledash($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and has a dash in the middle of the hex string.
+this appears in the pgsql documentation
+
+ 001122-334455
+
+=head2 mac_as_sun($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is B<not> padded and with B<-> delimiting every octet
+(i.e. after every 2nd character)
+
+ 0-34-56-78-9a-bc
+
+=head2 mac_as_tokenring($mac)
+
+Returns the mac address in $mac normalized as a hexadecimal string that is 0 padded and with B<-> delimiting every octet
+(i.e. after every 2nd character) and each octet is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
+
+ 00-2d-6a-1e-59-3d
+
 =head1 ERROR HANDLING
 
 Prior to 0.8 every error resulted in a die (croak) which needed to be caught.
 As I have used this module more, having to catch them all the time is tiresome.
-So from 0.8 onwards, errors result in an I<undef> and something being set.
+So from 0.8 onward, errors result in an I<undef> and something being set.
 
 For objects, this something is accessible via B<$self-E<gt>errstr> otherwise
-ther error is in B<$NetAddr::MAC::errstr>;
+the error is in B<$NetAddr::MAC::errstr>;
 
 If you would like to have die (croak) instead, you can either set the global
 B<$NetAddr::MAC::die_on_error> or set the B<die_on_error> option when creating
@@ -1510,6 +1529,15 @@ Or do it globally
 
 Stolen lots of ideas and some pod content from L<Device::MAC> and L<Net::MAC>
 
+=head1 SEE ALSO
+
+In some circumstances, the L<Regexp::Common::net> might be a slimmer solution.
+
+L<Net::MAC::Vendor> is useful for doing vendor look up.
+
+If you find L<Device::MAC> or L<Net::MAC> to be more suitable, please send
+feedback so I can improve!
+
 =head1 TODO
 
  - moare tests!
@@ -1524,19 +1552,10 @@ Many young people like to use Github, so by all means send me pull requests at
 
   https://github.com/djzort/NetAddr-MAC
 
-=head1 AUTHOR
-
-Dean Hamstead C<< <dean@bytefoundry.com.au> >>
-
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
-
 =head1 MOTIVATION
 
 There are lots of systems at my (then) place of work which handle MAC
-addresses. There was lots of code validating and normalising them all over
+addresses. There was lots of code validating and normalizing them all over
 the place - most of it was quirky and sloppy. So I set about creating a
 reusable module to add to our SOE install so that MAC address handling
 would become consistent, reliable, powerful and trivial.
@@ -1548,19 +1567,29 @@ There are several other MAC address modules on CPAN. I didn't like the
 interface on one, the other dragged in Moose. So I created this module,
 taking the ideas I liked from the other two modules and adding in extra bits
 that I needed (and a few features just for completeness) whilst avoiding
-dependancies and avoiding anything that doesnt work on perl 5.6
+dependencies and avoiding anything that doesn't work on perl 5.6
 
 I hope that the result is useful to others, the concept is to be able to create
 an object representing a MAC address based on a string that only very vaguely
-resembles a MAC address. From there, to be able to output normalised string
+resembles a MAC address. From there, to be able to output normalized string
 representations of the mac address in a variety of common formats.
 
 A templating function is deliberately omitted, as very niche outputs can easily
 be derived from the 'basic' format.
 
 Feel free to send patches for features you add, I appreciate those who
-have done so far and endeavour to incoporate new patches ASAP.
+have done so far and endeavour to incorporate new patches ASAP.
+
+=head1 AUTHOR
+
+Dean Hamstead <dean@fragfest.com.au>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2022 by Dean Hamstad.
+
+This is free software, licensed under:
+
+  The MIT (X11) License
 
 =cut
-
-1;
